﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using System.IO;
using System.Reflection;
using System.Xml.Linq;
using C1.WPF.DataGrid;
using System.Xml;

namespace WpfDataGridExcelExport
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
            c1DataGrid1.ItemsSource = GetProducts();
        }

        public static IEnumerable<Product> GetProducts()
        {
            IEnumerable<Product> products = Data.GetProducts((product) => !string.IsNullOrEmpty(product.Element("ProductModelID").Value));
            foreach (var product in products)
            {
                product.ExpirationDate = GetDate(product.StandardCost);
                product.Available = GetAvailable(product.Name);
            }
            return products;
        }

        private static bool? GetAvailable(string seed)
        {
            return (seed.Length % 3 == 0 ? (bool?)true : (seed.Length % 3 == 1 ? (bool?)false : (bool?)null));
        }

        private static DateTime? GetDate(double seed)
        {
            return new DateTime(DateTime.Now.Ticks - ((long)seed * 10000000000));
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new SaveFileDialog();

            dialog.DefaultExt = "*.xml";
            dialog.Filter = "Excel Xml (*.xml)|*.xml|All files (*.*)|*.*";

            if (dialog.ShowDialog() == false) return;

            using (var sw = new StreamWriter(dialog.OpenFile()))
            {
                var excelContent = GetExcelFile(true, false, true);
                excelContent.Save(sw);
            }
        }

        private XDocument GetExcelFile(bool generateColumnTitles, bool setRowHeight, bool setColumnWidth)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            string excelTemplateResourcestring = assembly.GetManifestResourceNames().First((resourceName) => resourceName.Contains(".Resources.ExcelTemplate.xml"));
            XDocument xml = XDocument.Load(XmlReader.Create(assembly.GetManifestResourceStream(excelTemplateResourcestring)));
            var table = (from element in xml.Descendants("{urn:schemas-microsoft-com:office:spreadsheet}Worksheet")
                         where element.Attribute("{urn:schemas-microsoft-com:office:spreadsheet}Name").Value == "Sheet1"
                         select element.Element("{urn:schemas-microsoft-com:office:spreadsheet}Table")).First();

            //table settings
            var orderedColumns = c1DataGrid1.Columns.OrderBy(column => column.DisplayIndex).ToList();
            table.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}ExpandedColumnCount", orderedColumns.Count);
            table.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}ExpandedRowCount", c1DataGrid1.Rows.Count + (generateColumnTitles ? 1 : 0));
            if (c1DataGrid1.RowHeight.IsAbsolute && setRowHeight)
            {
                table.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}DefaultRowHeight", c1DataGrid1.RowHeight.Value);
            }

            //add the rows
            for (int i = generateColumnTitles ? -1 : 0; i < c1DataGrid1.Rows.Count; i++)
            {
                DataGridRow row = null;
                if (i >= 0)
                {
                    row = c1DataGrid1.Rows[i];
                }
                XElement rowElement = new XElement("{urn:schemas-microsoft-com:office:spreadsheet}Row");
                bool isFirstColumn = true;
                //add the columns
                for (int j = 0; j < orderedColumns.Count; j++)
                {
                    DataGridColumn column = orderedColumns[j];
                    //If is the row that contains the column titles
                    if (i == -1)
                    {
                        string columnTitle = column.GetColumnText();
                        XElement columnElement = new XElement("{urn:schemas-microsoft-com:office:spreadsheet}Column");
                        columnElement.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}AutoFitWidth", column.Width.IsAbsolute ? 0 : 1);
                        if (column.Width.IsAbsolute && setColumnWidth)
                        {
                            columnElement.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}Width", column.Width.Value);
                        }
                        table.Add(columnElement);
                        XElement titleElement = new XElement("{urn:schemas-microsoft-com:office:spreadsheet}Cell");
                        XElement titleDataElement = new XElement("{urn:schemas-microsoft-com:office:spreadsheet}Data", columnTitle);
                        titleDataElement.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}Type", "String");
                        titleElement.Add(titleDataElement);
                        rowElement.Add(titleElement);
                    }
                    else
                    {
                        if (row.Type == DataGridRowType.Group)
                        {
                            if (isFirstColumn)
                            {
                                //output.Append(GetGroupText((DataGridGroupRow)row));
                            }
                        }
                        else
                        {
                            XElement cellElement = new XElement("{urn:schemas-microsoft-com:office:spreadsheet}Cell");
                            var cell = c1DataGrid1[row, column];
                            if (cell != null)
                            {
                                XElement cellDataElement = new XElement("{urn:schemas-microsoft-com:office:spreadsheet}Data", cell.Text);
                                cellDataElement.SetAttributeValue("{urn:schemas-microsoft-com:office:spreadsheet}Type", GetColumnExcelType(column));
                                cellElement.Add(cellDataElement);
                            }
                            rowElement.Add(cellElement);
                        }
                    }
                    isFirstColumn = false;
                }
                table.Add(rowElement);
            }
            return xml;
        }

        private static string GetColumnExcelType(DataGridColumn column)
        {
            if (column is DataGridNumericColumn)
            {
                return "Number";
            }
            return "String";
        }


    }

    public class Models : List<Model>
    {
        public Models()
        {
            foreach (var model in Data.GetModels())
            {
                Add(model);
            }
        }
    }
}
